@@ -34,6 +34,7 @@ from netperf_wrapper.build_info import DATA_DIR, VERSION
 from netperf_wrapper import util
 
 DEFAULT_SETTINGS = {
+    'NAME': None,
     'HOST': None,
     'HOSTS': [],
     'LOCAL_HOST': socket.gethostname(),
@@ -42,6 +43,7 @@ DEFAULT_SETTINGS = {
     'OUTPUT': '-',
     'FORMAT': 'default',
     'TITLE': '',
+    'NOTE': '',
     'LOG_FILE': None,
     'INPUT': [],
     'DESCRIPTION': 'No description',
@@ -50,6 +52,7 @@ DEFAULT_SETTINGS = {
     'DELAY': 5,
     'TIME': datetime.now(),
     'SCALE_DATA': [],
+    'SCALE_MODE': False,
     'ANNOTATE': True,
     'PRINT_TITLE': True,
     'PRINT_LEGEND': True,
@@ -61,7 +64,12 @@ TEST_PATH = os.path.join(DATA_DIR, 'tests')
 DICT_SETTINGS = ('DATA_SETS', 'PLOTS')
 
 def version(*args):
-    print("Netperf-wrapper v%s." %(VERSION))
+    print("Netperf-wrapper v%s.\nRunning on Python %s." %(VERSION, sys.version.replace("\n", " ")))
+    try:
+        import matplotlib
+        print("Using matplotlib version %s on numpy %s." % (matplotlib.__version__, matplotlib.__version__numpy__))
+    except ImportError:
+        print("No matplotlib found. Plots won't be available.")
     sys.exit(0)
 
 
@@ -110,7 +118,7 @@ class Glob(object):
 
 class TestEnvironment(object):
 
-    def __init__(self, env={}):
+    def __init__(self, env={}, informational=False):
         self.env = dict(env)
         self.env.update({
             'glob': Glob,
@@ -119,6 +127,7 @@ class TestEnvironment(object):
             'min_host_count': self.require_host_count,
             'find_ping': self.find_ping,
             })
+        self.informational = informational
 
     def execute(self, filename):
         try:
@@ -165,7 +174,9 @@ class TestEnvironment(object):
 
     def require_host_count(self, count):
         if len(self.env['HOSTS']) < count:
-            if 'DEFAULTS' in self.env and 'HOSTS' in self.env['DEFAULTS'] and self.env['DEFAULTS']['HOSTS']:
+            if self.informational:
+                self.env['HOSTS'] = ['dummy']*count
+            elif 'DEFAULTS' in self.env and 'HOSTS' in self.env['DEFAULTS'] and self.env['DEFAULTS']['HOSTS']:
                 # If a default HOSTS list is set, populate the HOSTS list with
                 # values from this list, repeating as necessary up to count
                 def_hosts = self.env['DEFAULTS']['HOSTS']
@@ -177,69 +188,114 @@ class TestEnvironment(object):
             else:
                 raise RuntimeError("Need %d hosts, only %d specified" % (count, len(self.env['HOSTS'])))
 
-parser = optparse.OptionParser(description='Wrapper to run concurrent netperf-style tests',
-                               usage="usage: %prog [options] -H <host> test")
+parser = optparse.OptionParser(description='Wrapper to run concurrent netperf-style tests.',
+                               usage="Usage: %prog [options] <host|test|input file ...> ")
 
 parser.add_option("-o", "--output", action="store", type="string", dest="OUTPUT",
-                  help="file to write output to (default standard out)")
+                  help="File to write processed output to (default standard out). The JSON "
+                  "data file is written to the same directory as this file, if provided. "
+                  "Otherwise, the data file is written to the current directory.")
 parser.add_option("-i", "--input", action="append", type="string", dest="INPUT",
-                  help="file to read input from (instead of running tests)")
+                  help="File to read input from (instead of running tests). Input files "
+                  "can also be specified as unqualified arguments without using the -i switch.")
 parser.add_option("-f", "--format", action="store", type="string", dest="FORMAT",
-                  help="select output format (plot, csv, org_table)")
+                  help="Select output format (plot, csv, org_table, stats). Default is no "
+                  "processed output (just writes the JSON data file).")
 parser.add_option("-p", "--plot", action="store", type="string", dest="PLOT",
-                  help="select which plot to output for the given test (implies -f plot)")
-parser.add_option("-H", "--host", action="append", type="string", dest="HOSTS", metavar='HOST',
-                  help="host to connect to for tests. Specify multiple hosts with multiple -H "
-                  "options (not all tests support this).")
+                  help="Select which plot to output for the given test (implies -f plot). "
+                  "Use the --list-plots option to see available plots.")
 parser.add_option("-t", "--title-extra", action="store", type="string", dest="TITLE",
-                  help="text to add to plot title")
-parser.add_option("-L", "--log-file", action="store", type="string", dest="LOG_FILE",
-                  help="write debug log (test program output) to log file")
-parser.add_option("-l", "--length", action="store", type="int", dest="LENGTH",
-                  help="base test length (some tests may add some time to this)")
-parser.add_option("-s", "--step-size", action="store", type="float", dest="STEP_SIZE",
-                  help="measurement data point step size")
-parser.add_option("-d", "--delay", action="store", type="int", dest="DELAY",
-                  help="number of seconds to delay second parts of test (such as bandwidth loaders)")
-parser.add_option("-4", "--ipv4", action="store_const", const=4, dest="IP_VERSION",
-                  help="use IPv4 for tests (some tests may ignore this)")
-parser.add_option("-6", "--ipv6", action="store_const", const=6, dest="IP_VERSION",
-                  help="use IPv6 for tests (some tests may ignore this)")
+                  help="Text to add to plot title and data file name.")
+parser.add_option("-n", "--note", action="store", type="string", dest="NOTE",
+                  help="Add arbitrary text as a note to be stored in the JSON data file "
+                  "(under the NOTE key in the metadata object).")
 
-parser.add_option("-V", "--version", action="callback", callback=version,
-                  help="show netperf-wrapper version and exit")
 
-parser.add_option("-z", "--zero-y", action="store_true", dest="ZERO_Y",
-                  help="start y axis of plot at zero (also disables log scales)")
-parser.add_option("--disable-log", action="store_false", dest="LOG_SCALE",
-                  help="disable log scales on plots")
-parser.add_option('--list-tests', action='store_true', dest="LIST_TESTS",
-                  help="list available tests")
-parser.add_option('--list-plots', action='store_true', dest="LIST_PLOTS",
-                  help="list available plots for selected test")
-parser.add_option("--scale-data", action="append", type="string", dest="SCALE_DATA",
-                  help="additional data files to use for scaling the plot axes "
-                  "(can be supplied multiple times)")
-parser.add_option("--no-annotation", action="store_false", dest="ANNOTATE",
-                  help="do not annotate plots with hosts, time and test length")
-parser.add_option("--no-legend", action="store_false", dest="PRINT_LEGEND",
-                  help="do not print plot legend")
-parser.add_option("--no-title", action="store_false", dest="PRINT_TITLE",
-                  help="do not print plot title")
+test_group = optparse.OptionGroup(parser, "Test configuration",
+                                  "These options affect the behaviour of the test being run "
+                                  "and have no effect when parsing input files.")
+test_group.add_option("-H", "--host", action="append", type="string", dest="HOSTS", metavar='HOST',
+                  help="Host to connect to for tests. For tests that support it, multiple hosts "
+                  "can be specified by supplying this option multiple times. Hosts can also be "
+                  "specified as unqualified arguments; this parameter guarantees that the "
+                  "argument be interpreted as a host name (rather than being subject to "
+                  "auto-detection between input files, hostnames and test names).")
+test_group.add_option("-l", "--length", action="store", type="int", dest="LENGTH",
+                  help="Base test length (some tests may add some time to this).")
+test_group.add_option("-s", "--step-size", action="store", type="float", dest="STEP_SIZE",
+                  help="Measurement data point step size.")
+test_group.add_option("-d", "--delay", action="store", type="int", dest="DELAY",
+                  help="Number of seconds to delay parts of test (such as bandwidth "
+                  "loaders).")
+test_group.add_option("-4", "--ipv4", action="store_const", const=4, dest="IP_VERSION",
+                  help="Use IPv4 for tests (some tests may ignore this).")
+test_group.add_option("-6", "--ipv6", action="store_const", const=6, dest="IP_VERSION",
+                  help="Use IPv6 for tests (some tests may ignore this).")
+parser.add_option_group(test_group)
+
+plot_group = optparse.OptionGroup(parser, "Plot configuration",
+                                  "These options are used to configure the appearance of "
+                                  "plot output and only make sense combined with -f plot.")
+
+plot_group.add_option("-z", "--zero-y", action="store_true", dest="ZERO_Y",
+                  help="Always start y axis of plot at zero, instead of auto-scaling the "
+                  "axis (also disables log scales). Auto-scaling is still enabled for the "
+                  "upper bound.")
+plot_group.add_option("--disable-log", action="store_false", dest="LOG_SCALE",
+                  help="Disable log scales on plots.")
+plot_group.add_option("--scale-data", action="append", type="string", dest="SCALE_DATA",
+                  help="Additional data files to consider when scaling the plot axes "
+                  "(for plotting several plots with identical axes). "
+                  "Can be supplied multiple times; see also --scale-mode.")
+plot_group.add_option("-S", "--scale-mode", action="store_true", dest="SCALE_MODE",
+                  help="Treat file names (except for the first one) passed as unqualified "
+                  "arguments as if passed as --scale-data (default as if passed as --input).")
+plot_group.add_option("--no-annotation", action="store_false", dest="ANNOTATE",
+                  help="Exclude annotation with hostnames, time and test length from plots.")
+plot_group.add_option("--no-legend", action="store_false", dest="PRINT_LEGEND",
+                  help="Exclude legend from plots.")
+plot_group.add_option("--no-title", action="store_false", dest="PRINT_TITLE",
+                  help="Exclude title from plots.")
+parser.add_option_group(plot_group)
+
+
+misc_group = optparse.OptionGroup(parser, "Misc and debugging options")
+misc_group.add_option("-L", "--log-file", action="store", type="string", dest="LOG_FILE",
+                  help="Write debug log (test program output) to log file.")
+misc_group.add_option('--list-tests', action='store_true', dest="LIST_TESTS",
+                  help="List available tests and exit.")
+misc_group.add_option('--list-plots', action='store_true', dest="LIST_PLOTS",
+                  help="List available plots for selected test and exit.")
+misc_group.add_option("-V", "--version", action="callback", callback=version,
+                  help="Show netperf-wrapper version information and exit.")
+parser.add_option_group(misc_group)
 
 class Settings(optparse.Values, object):
 
-    def load_test(self, test_name):
-        self.NAME = test_name
+    def check_testname(self, test_name):
+        filename = os.path.join(TEST_PATH, test_name + ".conf")
+
+        if not os.path.exists(filename):
+            # Test not found, assume it's a hostname
+            self.HOSTS.append(test_name)
+        elif self.NAME is not None and self.NAME != test_name:
+            raise RuntimeError("Multiple test names specified.")
+        else:
+            self.NAME = test_name
+
+
+    def load_test(self, test_name=None, informational=False):
+        if test_name is not None:
+            self.NAME=test_name
+        if self.NAME is None:
+            raise RuntimeError("Missing test name.")
         if self.HOSTS:
             self.HOST = self.HOSTS[0]
 
         self.lookup_hosts()
 
-        test_env = TestEnvironment(self.__dict__)
-        filename = os.path.join(TEST_PATH, test_name + ".conf")
-        if not os.path.exists(filename):
-            raise RuntimeError("No config file found for test '%s'" % test_name)
+        test_env = TestEnvironment(self.__dict__, informational)
+        filename = os.path.join(TEST_PATH, self.NAME + ".conf")
         s = test_env.execute(filename)
 
         for k,v in list(s.items()):
@@ -294,6 +350,15 @@ def load():
     if hasattr(settings, 'PLOT'):
         settings.FORMAT = 'plot'
 
+    for a in args:
+        if os.path.exists(a):
+            if settings.SCALE_MODE and settings.INPUT:
+                settings.SCALE_DATA.append(a)
+            else:
+                settings.INPUT.append(a)
+        else:
+            settings.check_testname(a)
+
     if settings.INPUT:
         results = []
         test_name = None
@@ -306,20 +371,16 @@ def load():
 
 
         settings.update(results[0].meta())
-        settings.load_test(test_name)
+        settings.load_test()
     else:
-        if len(args) < 1:
-            parser.error("Missing test name.")
-
-        test_name = args[0]
-
-        settings.load_test(test_name)
+        settings.load_test()
         results = [ResultSet(NAME=settings.NAME,
                             HOST=settings.HOST,
                             HOSTS=settings.HOSTS,
                             TIME=settings.TIME,
                             LOCAL_HOST=settings.LOCAL_HOST,
                             TITLE=settings.TITLE,
+                            NOTE=settings.NOTE,
                             LENGTH=settings.LENGTH,
                             TOTAL_LENGTH=settings.TOTAL_LENGTH,
                             STEP_SIZE=settings.STEP_SIZE,)]
@@ -351,11 +412,12 @@ def load():
 def list_tests():
     tests = sorted([os.path.splitext(i)[0] for i in os.listdir(TEST_PATH) if i.endswith('.conf')])
     sys.stderr.write('Available tests:\n')
-    max_len = str(max([len(t) for t in tests]))
+    max_len = max([len(t) for t in tests])
     for t in tests:
         settings.update(DEFAULT_SETTINGS)
-        settings.load_test(t)
-        sys.stderr.write(("  %-"+max_len+"s :  %s\n") % (t, settings.DESCRIPTION))
+        settings.load_test(t, informational=True)
+        desc = settings.DESCRIPTION.replace("\n", "\n"+" "*(max_len+6))
+        sys.stderr.write(("  %-"+str(max_len)+"s :  %s\n") % (t, desc))
     sys.exit(0)
 
 def list_plots():
